# -*- coding: utf-8 -*-

import os
import logging
import tempfile
import ndexutil
import shutil
import json
from ndexutil.argparseutil import ArgParseFormatter
from ndexutil.config import NDExUtilConfig
from ndexutil.ndex import NDExExtraUtils
from ndexutil.exceptions import NDExUtilError
from ndex2.client import Ndex2
from ndex2.nice_cx_network import DefaultNetworkXFactory
import ndex2
import networkx
from requests.exceptions import HTTPError

# create logger
logger = logging.getLogger('ndexutil.networkx')


SPRING_LAYOUT = 'spring'
"""
Spring layout
"""

CIRCULAR_LAYOUT = 'circular'
"""
Circular layout
"""

KAMADA_KAWAI_LAYOUT = 'kamada_kawai'
"""
Kamada Kawai layout
"""

PLANAR_LAYOUT = 'planar'
"""
Planar layout
"""

SHELL_LAYOUT = 'shell'
"""
Shell layout
"""

SPECTRAL_LAYOUT = 'spectral'
"""
Spectral layout
"""

SPIRAL_LAYOUT = 'spiral'
"""
Spiral layout
"""


class NetworkxLayoutWrapper(object):
    """
    Wrapper for networkx layout algorithms
    """
    SPRING_ITERATIONS = 50
    SPRING_K = None

    def __init__(self):
        """
        Constructor
        """
        pass

    def spring_layout(self, g, k=None, pos=None, fixed=None,
                      iterations=50, threshold=0.0001,
                      weight='weight', scale=1,
                      center=None, dim=2, seed=None):
        """
        Applies networkx spring layout on graph
        See :py:class:`networkx.drawing.layout.spring_layout` for usage

        :param g: Graph to apply layout on
        :type g: :py:class:`~networkx.Graph`
        :return: positions for nodes generated by layout algorithm
        :rtype: dict
        """
        if g is None:
            raise NDExUtilError('Graph passed in is None')
        return networkx.drawing.spring_layout(g,
                                              k=k, pos=pos, fixed=fixed,
                                              iterations=iterations,
                                              threshold=threshold,
                                              weight=weight, scale=scale,
                                              center=center, dim=dim, seed=seed)

    def convert_positions_to_cartesian_aspect(self, networkx_pos=None):
        """
        Converts node coordinates from a pos object
        to a list of dicts with following format:
        [{'node': <node id>,
          'x': <x position>,
          'y': <y position>}]

        :param networkx_pos: dict of node coordinates from networkx layout
                             algorithm. Which should be in format of:
                             {NODEID: [X, Y],...}
        :type networkx_pos: dict
        :return: coordinates as list of dict objects in format:
                 [{'node': ID,'x': X,'y': Y}...
        :rtype: list
        """
        if networkx_pos is None:
            raise NDExUtilError('networkx_pos is None')

        return [{'node': n,
                 'x': float(networkx_pos[n][0]),
                 'y': float(networkx_pos[n][1])} for n in networkx_pos]

    def get_center_as_list(self, center=None):
        """
        Parses string passed into 'center' that should be
        coordinates with comma delimiter (example: X,Y),
        this method returns a list of each element

        :param center: coordinates delimited by comma
        :type center: str
        :return: list of x,y coordinate of center or `None` if value is `None`
        :rtype: list
        """
        if center is None:
            return None
        split_val = center.split(',')
        if len(split_val) != 2:
            raise NDExUtilError('Expecting single comma delimited '
                                'string. Value passed in '
                                'must be two numbers '
                                'delimited by comma ie 4,5')
        try:
            return [float(split_val[0]), float(split_val[1])]
        except Exception as e:
            raise NDExUtilError('Value passed in must be two'
                                'numbers delimited by comma: ' +
                                str(e))

    def run_layout_algorithm(self, layout=None,
                             netx_graph=None,
                             center=None,
                             scale=None,
                             arg_dict=None):
        """
        Runs layout specified by `layout`
        on network `netx_graph`

        :param layout: Layout to run, currently only spring is supported
        :type layout: str
        :param netx_graph: Network to run the layout on
        :type netx_graph: :py:class:`~networkx.Graph`
        :param center: Comma delimited center coordinates to pass
                       to layout algorithm
        :type center: str
        :param scale: Scale to pass to layout algorithm
        :type scale: float
        :param arg_dict: Layout specific parameters. For spring the following
                         two parameters are supported: 'spring_k' and '
                         spring_iterations'
                         which should be set to float and int respectively.
                         For example:
                         {'spring_k': 0.5, 'spring_iterations':=10}
        :type arg_dict: dict
        :raises NDExUtilError: If layout is None or does not match
                               supported layouts
        :return: positions generated by layout algorithm
        :rtype: dict
        """
        if layout is None:
            raise NDExUtilError('Layout cannot be None')

        center_val = self.get_center_as_list(center=center)
        if layout == SPRING_LAYOUT:
            k_val = NetworkxLayoutWrapper.SPRING_K
            k_iter = NetworkxLayoutWrapper.SPRING_ITERATIONS
            if arg_dict is not None:
                if 'spring_k' in arg_dict:
                    k_val = arg_dict['spring_k']
                if 'spring_iterations' in arg_dict:
                    k_iter = arg_dict['spring_iterations']

            return self.spring_layout(netx_graph,
                                      k=k_val,
                                      iterations=k_iter,
                                      center=center_val,
                                      scale=scale)
        raise NDExUtilError(str(layout) +
                            ' does not match supported layout')

    def apply_layout(self, input_cx_file=None, layout=None,
                     output_cx_file=None, center=None,
                     scale=None, arg_dict=None):
        """
        Apply layout via `networkx <http://networkx.org/documentation/stable/>`_ on network specified by
        cxfile

        :param layout: Layout to run, currently only spring is supported
        :type layout: str
        :param input_cx_file: Path to CX file of network to generate layout for
        :type input_cx_file: str
        :param output_cx_file: Path to write updated CX file
        :type output_cx_file: str
        :param center: Comma delimited center coordinates to pass
                       to layout algorithm
        :type center: str
        :param scale: Scale to pass to layout algorithm
        :type scale: float
        :param arg_dict: Layout specific parameters. For spring the following
                         two parameters are supported: 'spring_k' and '
                         spring_iterations'
                         which should be set to float and int respectively.
                         For example:
                         {'spring_k': 0.5, 'spring_iterations':=10}
        :type arg_dict: dict
        :raises NDExUtilError: If layout is None or does not match
                               supported layouts
        :return: (`cartesianLayout` aspect, path to CX file with layout)
        :rtype: tuple
        """
        if layout is None:
            raise NDExUtilError('Layout cannot be None')

        logger.info('Loading network')
        net = ndex2.create_nice_cx_from_file(input_cx_file)
        netx_fac = DefaultNetworkXFactory()
        netx_graph = netx_fac.get_graph(net)

        logger.info('Applying Networkx ' +
                    layout + ' network on network')

        pos = self.run_layout_algorithm(layout=layout, netx_graph=netx_graph,
                                        center=center,
                                        scale=scale, arg_dict=arg_dict)

        del netx_graph

        logger.debug('Converting coordinates from networkx to CX format')
        cart_aspect = self.convert_positions_to_cartesian_aspect(pos)

        logger.info('Writing out CX file: ' + output_cx_file)
        net.set_opaque_aspect('cartesianLayout', cart_aspect)
        with open(output_cx_file, 'w') as f:
            json.dump(net.to_cx(), f)

        return cart_aspect, output_cx_file


class NetworkxLayoutCommand(object):
    """
    Updates network in NDEx with Networkx layout
    """
    COMMAND = 'networkxlayout'

    def __init__(self, theargs,
                 layout_wrapper=NetworkxLayoutWrapper(),
                 ndexextra=NDExExtraUtils(),
                 altclient=None):
        """
        Constructor

        :param theargs: command line arguments from Argparse
                        ie theargs.name theargs.type
        """
        self._args = theargs
        self._user = self._args.username
        self._pass = self._args.password
        self._server = self._args.server
        self._tmpdir = None  # set in run() function
        self._layoutwrapper = layout_wrapper
        self._ndexextra = ndexextra
        self._altclient = altclient

    def _parse_config(self):
        """
        Parses user, password, and server from command line
        positional flags unless those values are '-' in which
        case those values are extracted from config
        specified by --profile flag

        If --profile is used then these values are extracted
        from config:

        :py:const:`~ndexutil.config.NDExUtilConfig.USER`
        :py:const:`~ndexutil.config.NDExUtilConfig.PASSWORD`
        :py:const:`~ndexutil.config.NDExUtilConfig.SERVER`

        :return: None
        """
        if self._user != '-' and self._pass != '-' and self._server != '-':
            return

        ncon = NDExUtilConfig(conf_file=self._args.conf)
        con = ncon.get_config()
        if self._user == '-':
            self._user = con.get(self._args.profile, NDExUtilConfig.USER)

        if self._pass == '-':
            self._pass = con.get(self._args.profile, NDExUtilConfig.PASSWORD)

        if self._server == '-':
            self._server = con.get(self._args.profile, NDExUtilConfig.SERVER)

    def _get_client(self):
        """
        Gets Ndex2 client by parsing command line flags or extracting
        values from --profile flag

        :return: Ndex2 python client
        :rtype: :py:class:`~ndex2.client.Ndex2`
        """
        if self._altclient is not None:
            return self._altclient
        return Ndex2(self._server, self._user, self._pass)

    def run(self):
        """
        Retrieves network specified by --uuid, applies layout
        specified via --layout flag and optionally updates layout
        or entire network on NDEx

        :raises NDExUtilError if there is an error
        :return: 0 upon success otherwise failure
        """

        logger.warning('THIS IS AN UNTESTED ALPHA IMPLEMENTATION '
                       'AND MAY CONTAIN ERRORS')

        self._parse_config()
        client = self._get_client()
        self._tmpdir = tempfile.mkdtemp(prefix=self._args.tmpdir)
        try:
            input_cx_file = os.path.join(self._tmpdir, self._args.uuid + '.cx')
            self._ndexextra.download_network_from_ndex(client=client,
                                                       networkid=self._args.uuid,
                                                       destfile=input_cx_file)

            aspect_data, output_cx_file = self.apply_layout(cxfile=input_cx_file)

            if self._args.skipupload is True:
                logger.info('Skipping upload to NDEx')
                return 0

            if self._args.updatefullnetwork is True:
                logger.info('Updating entire network with id: ' +
                            str(self._args.uuid) + ' on NDEx server: ' +
                            str(self._server) +
                            ' since --updatenetwork flag is set')
                self._ndexextra.update_network_on_ndex(client=client,
                                                       networkid=self._args.uuid,
                                                       cxfile=output_cx_file)
            else:
                self._ndexextra.update_network_aspect_on_ndex(client=client,
                                                              networkid=self._args.uuid,
                                                              aspect_name='cartesianLayout',
                                                              aspect_data=aspect_data)
            return 0
        except HTTPError as he:
            logger.fatal('Received error code: ' +
                         str(he.response.status_code) +
                         ' from NDEx server', he)
            if 'message' in he.response.json():
                logger.fatal('Message from NDEx server: ' + str(he.response.json()['message']))
            return 1
        finally:
            shutil.rmtree(self._tmpdir)

    def get_center_as_list(self):
        """
        Parses string passed into --center that should be
        coordinates with comma delimiter (example: X,Y),
        this method returns a list of each element

        :return: list of x,y coordinate of center or `None` if value is `None`
        :rtype: list
        """
        if self._args.center is None:
            return None
        split_val = self._args.center.split(',')
        if len(split_val) != 2:
            raise NDExUtilError('Expecting single comma delimited '
                                'string. Value passed to --center '
                                'must be two numbers '
                                'delimited by comma ie 4,5')
        try:
            return [float(split_val[0]), float(split_val[1])]
        except Exception as e:
            raise NDExUtilError('Value passed to --center must be two'
                                'numbers delimited by comma: ' +
                                str(e))

    def _run_layout_algorithm(self, netx_graph=None):
        """
        Runs layout specified by <layout> command line flag
        on network `netx_graph`

        :param netx_graph: Network to run the layout on
        :type netx_graph: :py:class:`~networkx.Graph`
        :return: positions generated by layout algorithm
        :rtype: dict
        """
        center_val = self.get_center_as_list()
        if self._args.layout == SPRING_LAYOUT:
            return self._layoutwrapper.spring_layout(netx_graph,
                                                     k=self._args.spring_k,
                                                     iterations=self._args.spring_iterations,
                                                     center=center_val,
                                                     scale=self._args.scale)

        raise NDExUtilError(str(self._args.layout) +
                            ' does not match supported layout')

    def apply_layout(self, cxfile=None):
        """
        Apply layout via networkx on network specified by
        cxfile

        :param cxfile: Path to CX file of network to generate layout for
        :type cxfile: str
        :return: (cartesianLayout aspect, path to CX file with layout)
        :rtype: tuple
        """

        logger.info('Loading network')
        net = ndex2.create_nice_cx_from_file(cxfile)
        netx_fac = DefaultNetworkXFactory()
        netx_graph = netx_fac.get_graph(net)

        logger.info('Applying Networkx ' +
                    self._args.layout +
                    ' network on network')

        pos = self._run_layout_algorithm(netx_graph=netx_graph)

        del netx_graph

        logger.debug('Converting coordinates from networkx to CX format')
        cart_aspect = self._layoutwrapper.convert_positions_to_cartesian_aspect(pos)

        if self._args.outputcx is not None:
            output_cx_file = self._args.outputcx
        else:
            output_cx_file = os.path.join(self._tmpdir, 'output.cx')
        logger.info('Writing out CX file: ' + output_cx_file)
        net.set_opaque_aspect('cartesianLayout', cart_aspect)
        with open(output_cx_file, 'w') as f:
            json.dump(net.to_cx(), f)

        return cart_aspect, output_cx_file

    @staticmethod
    def add_subparser(subparsers):
        """
        adds a subparser
        :param subparsers:
        :return:
        """
        desc = """

Version {version}

The {cmd} command updates layout on a network in NDEx using Networkx.
The network must be specified by NDEx UUID via --uuid flag. 

By default this command only updates the cartesianLayout 
aspect. To update full network add --updatefullnetwork 
flag to command line.

The flags --scale and --center work for all layouts. 

Some flags only are relevant for certain layouts. 
Layout specific flags will start with --<LAYOUT NAME>_<FLAG> 
like --spring_k and --spring_iterations flags.

Upon success script will exit with code 0 otherwise error.

Example:

# to load NDEx connection info from [foo] profile in ~/.ndexutil.conf
file add --profile flag and use - - - in lieu of user, password, and server
ndexmisctools.py --profile foo networkxlayout spring - - - --uuid XXXX-XXX --spring_k 0.5

# to specify NDEx connection info on command line
ndexmisctools.py networkxlayout spring jdoe badpassword public.ndexbio.org --uuid XXXX-XXX


WARNING: THIS IS AN UNTESTED ALPHA IMPLEMENTATION AND MAY CONTAIN
         ERRORS. YOU HAVE BEEN WARNED.

        """.format(version=ndexutil.__version__,
                   cmd=NetworkxLayoutCommand.COMMAND)

        parser = subparsers.add_parser(NetworkxLayoutCommand.COMMAND,
                                       help='Updates layout of network via '
                                            'Cytoscape',
                                       description=desc,
                                       formatter_class=ArgParseFormatter)
        parser.add_argument('layout', choices=[SPRING_LAYOUT],
                            help='Name of layout to run.')
        parser.add_argument('username', help='NDEx username, if set to - '
                                             'then value from config will '
                                             'be used')
        parser.add_argument('password', help='NDEx password, if set to - '
                                             'then value from config will '
                                             'be used')
        parser.add_argument('server', help='NDEx server, if set to - then '
                                           'value from config will be used,'
                                           'For production, use public.ndexbio.org')
        parser.add_argument('--uuid', required=True,
                            help='The UUID of network in NDEx to update')
        parser.add_argument('--scale', type=float, default=300.0,
                            help='Scale to pass to layout algorithm.')
        parser.add_argument('--center', type=str,
                            help='Comma delimited coordinate denoting '
                                 'center for layout. Should be in format '
                                 'of X,Y or Y,X not sure which way networkx'
                                 'does coordinates')
        parser.add_argument('--' + SPRING_LAYOUT + '_iterations', type=int,
                            default=50,
                            help='Maximum number of iterations taken ')
        parser.add_argument('--' + SPRING_LAYOUT + '_k', type=float,
                            help='Optimal distance between nodes. '
                                 'If unset the distance is set to 1/sqrt(n) '
                                 'where n is the number of nodes. Increase '
                                 'this value to move nodes farther apart.')
        parser.add_argument('--tmpdir',
                            help='Sets temp directory used for processing. If '
                                 'not set, then directory used is the '
                                 'default for Python\'s '
                                 'tempfile.mkdtemp() function')
        parser.add_argument('--skipupload', action='store_true',
                            help='If set, layout will NOT updated for '
                                 'network in NDEx')
        parser.add_argument('--outputcx',
                            help='If set, CX will be written to this file')
        parser.add_argument('--updatefullnetwork', action='store_true',
                            help='If set, update entire network instead '
                                 'of just the cartesianLayout aspect')
        return parser



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ndex` module."""

import tempfile
import shutil
import os
import json
import unittest
from unittest.mock import call, MagicMock

from ndex2.nice_cx_network import NiceCXNetwork
from ndexutil.networkx import NetworkxLayoutCommand
from ndexutil.config import NDExUtilConfig
from ndexutil.exceptions import ConfigError
from ndexutil.exceptions import NDExUtilError


class TestNetworkxLayoutWrapper(unittest.TestCase):
    """
        Tests NDExExtraUtils in ndex module
    """
    def setUp(self):
        """Set up test fixtures, if any."""
        pass

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def get_dummy_configfile(self, temp_dir):
        """
        Given a base temp directory create a config file
        :param temp_dir:
        :return: full path to the config file
        """
        cfile = os.path.join(temp_dir, 'configfile')
        with open(cfile, 'w') as f:
            f.write('[foo]\n')
            f.write(NDExUtilConfig.USER + ' = theuser\n')
            f.write(NDExUtilConfig.PASSWORD + ' = thepass\n')
            f.write(NDExUtilConfig.SERVER + ' = theserver\n')
        return cfile

    def test_parse_config_no_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = MagicMock()
            p.username = 'bob'
            p.password = '-'
            p.server = 'public.ndexbio.org'
            p.conf = temp_dir
            remover = NetworkxLayoutCommand(p)
            remover._parse_config()
            self.fail('Expected ConfigError')
        except ConfigError as e:
            self.assertEqual('No configuration file found', str(e))
        finally:
            shutil.rmtree(temp_dir)

    def test_parse_config_valid_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            cfile = self.get_dummy_configfile(temp_dir)

            # try with username set to - to load it from config
            p = MagicMock()
            p.conf = cfile
            p.username = '-'
            p.password = '-'
            p.server = '-'
            p.profile = 'foo'
            loader = NetworkxLayoutCommand(p)
            loader._parse_config()
            self.assertEqual('theuser', loader._user)
            self.assertEqual('thepass', loader._pass)
            self.assertEqual('theserver', loader._server)
        finally:
            shutil.rmtree(temp_dir)

    def test_parse_config_all_set_via_cmdline(self):
        temp_dir = tempfile.mkdtemp()
        try:
            cfile = self.get_dummy_configfile(temp_dir)

            # try with username set to - to load it from config
            p = MagicMock()
            p.conf = cfile
            p.username = 'u'
            p.password = 'p'
            p.server = 's'
            loader = NetworkxLayoutCommand(p)
            loader._parse_config()
            self.assertEqual('u', loader._user)
            self.assertEqual('p', loader._pass)
            self.assertEqual('s', loader._server)
        finally:
            shutil.rmtree(temp_dir)

    def test_add_subparsers(self):
        mock_sub = MagicMock()
        mock_parse = MagicMock()
        mock_sub.add_parser = MagicMock(return_value=mock_parse)
        NetworkxLayoutCommand.add_subparser(mock_sub)

        mock_parse.add_argument.assert_has_calls([call('layout',
                                                       choices=['spring'],
                                                       help='Name of layout to run.')],
                                                       any_order=True)

    def test_get_center_as_list(self):
        p = MagicMock()
        p.username = 'u'
        p.password = 'p'
        p.server = 's'
        p.center = None

        cmd = NetworkxLayoutCommand(p)
        self.assertEquals(None, cmd.get_center_as_list())

        p.center = '123'
        try:
            cmd.get_center_as_list()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue(str(ne).startswith('Expecting single comma'))

        p.center = '4,5'
        self.assertEquals([4, 5], cmd.get_center_as_list())

        p.center = 'x,23'
        try:
            cmd.get_center_as_list()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue(str(ne).startswith('Value passed to --center'))

        p.center = ',10'
        try:
            cmd.get_center_as_list()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue(str(ne).startswith('Value passed to --center'))

        p.center = ',10,12'
        try:
            cmd.get_center_as_list()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue(str(ne).startswith('Expecting single comma'))

    def test_run_layout_algorithm_invalid_layout(self):
        p = MagicMock()
        p.username = 'u'
        p.password = 'p'
        p.server = 's'
        p.layout = 'nope'
        p.center = None
        cmd = NetworkxLayoutCommand(p)
        try:
            cmd._run_layout_algorithm(netx_graph='graph')
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('nope does not match supported layout', str(ne))

    def test_run_layout_algorithm(self):
        p = MagicMock()
        p.username = 'u'
        p.password = 'p'
        p.server = 's'
        p.center = None
        p.layout = 'spring'
        p.spring_k = 1.0
        p.spring_iterations = 50
        p.scale = 300.0

        layout = MagicMock()
        layout.spring_layout = MagicMock(return_value='result')

        cmd = NetworkxLayoutCommand(p,
                                    layout_wrapper=layout)
        res = cmd._run_layout_algorithm(netx_graph='graph')
        self.assertEqual('result', res)
        layout.spring_layout.assert_called_with('graph',
                                                k=1.0,
                                                iterations=50,
                                                center=None,
                                                scale=300.0)

    def test_apply_layout(self):
        p = MagicMock()
        p.username = 'u'
        p.password = 'p'
        p.server = 's'
        p.center = '3,2'
        p.layout = 'spring'
        p.spring_k = 1.0
        p.spring_iterations = 2
        p.scale = 300.0
        p.outputcx = None

        temp_dir = tempfile.mkdtemp()
        try:
            cxfile = os.path.join(temp_dir, 'input.cx')
            net = NiceCXNetwork()
            node_one = net.create_node('node1', 'rep1')
            node_two = net.create_node('node2', 'rep2')
            net.create_edge(node_one, node_two, 'links')
            with open(cxfile, 'w') as f:
                json.dump(net.to_cx(), f)
            cmd = NetworkxLayoutCommand(p)
            cmd._tmpdir = temp_dir
            c_a, outfile = cmd.apply_layout(cxfile=cxfile)
            self.assertTrue(os.path.isfile(outfile))
            self.assertEqual(2, len(c_a))
        finally:
            shutil.rmtree(temp_dir)

    def test_apply_layout_outputcx_set(self):
        temp_dir = tempfile.mkdtemp()
        p = MagicMock()
        p.username = 'u'
        p.password = 'p'
        p.server = 's'
        p.center = None
        p.layout = 'spring'
        p.spring_k = 1.0
        p.spring_iterations = 2
        p.scale = 300.0
        ocx = os.path.join(temp_dir, 'out.cx')
        p.outputcx = ocx

        try:
            cxfile = os.path.join(temp_dir, 'input.cx')
            net = NiceCXNetwork()
            node_one = net.create_node('node1', 'rep1')
            node_two = net.create_node('node2', 'rep2')
            net.create_edge(node_one, node_two, 'links')
            with open(cxfile, 'w') as f:
                json.dump(net.to_cx(), f)
            cmd = NetworkxLayoutCommand(p)
            cmd._tmpdir = temp_dir
            c_a, outfile = cmd.apply_layout(cxfile=cxfile)
            self.assertEqual(ocx, outfile)
            self.assertTrue(os.path.isfile(outfile))
            self.assertEqual(2, len(c_a))
        finally:
            shutil.rmtree(temp_dir)

    def test_run(self):
        temp_dir = tempfile.mkdtemp()
        p = MagicMock()
        p.tmpdir = None
        p.username = 'u'
        p.password = 'p'
        p.server = 's'
        p.center = None
        p.layout = 'spring'
        p.spring_k = 1.0
        p.spring_iterations = 2
        p.scale = 300.0
        p.uuid = 'uuid'
        p.skipupload = False
        p.updatefullnetwork = False

        mockextra = MagicMock()
        mocklayout = MagicMock()
        mockclient = MagicMock()
        cmd = NetworkxLayoutCommand(p, ndexextra=mockextra,
                                    layout_wrapper=mocklayout,
                                    altclient=mockclient)

        try:
            cmd.run()
            self.fail('Expected exception')
        except Exception as e:
            pass



# -*- coding: utf-8 -*-

import re
import logging
import requests
from requests.exceptions import HTTPError
from biothings_client import get_client

logger = logging.getLogger(__name__)


class NetworkIssueReport(object):
    """
    Holds summary information about issues found during network
    creation
    """
    def __init__(self, network_name):
        """
        Constructor

        :param network_name: name of network to display in report
        :type network_name: string
        """
        self._networkname = network_name
        self._issuemap = {}
        self._nodetype = set()

    def add_nodetype(self, nodetype):
        """
        Adds `nodetype` to set of node types

        :param nodetype: value of type node attribute
        :type nodetype: string
        :return: None
        """
        if nodetype is None:
            return
        self._nodetype.add(nodetype)

    def get_nodetypes(self):
        """
        Gets node types

        :return: set of node types
        :rtype: set
        """
        return self._nodetype

    def addissues(self, description, issue_list):
        """
        Adds issues to the report

        :param description: description of issue
        :type description: string
        :param issue_list: list of strings describing the issues
        :type issue_list: list
        :return: None
        """
        if issue_list is None:
            return
        if len(issue_list) is 0:
            return
        if description is None:
            return
        self._issuemap[description] = issue_list

    def get_fullreport_as_string(self):
        """
        Gets report as string

        :return: report in a human readable form with newlines
                 and tabs for indenting the issues
        :rtype: string
        """
        res = ''
        for key in self._issuemap.keys():
            num_issues = len(self._issuemap[key])
            if num_issues == 1:
                issue_word = 'issue'
            else:
                issue_word = 'issues'
            res += '\t' + str(num_issues) + ' ' + issue_word + ' -- ' +\
                   key + '\n'
            for entry in self._issuemap[key]:
                res += '\t\t' + str(entry) + '\n'
        if len(res) is 0:
            return ''

        return str(self._networkname) + '\n' + res


class NetworkUpdator(object):
    """
    Base class for classes that update
    a network
    """
    def __init__(self):
        """
        Constructor
        """
        pass

    def get_description(self):
        """
        Subclasses should implement
        :return:
        """
        raise NotImplementedError('subclasses should implement')

    def update(self, network):
        """
        subclasses should implement
        :param network:
        :return:
        """
        raise NotImplementedError('subclasses should implement')


class GeneSymbolSearcher(object):
    """
    Wrapper around :py:mod:`biothings_client` to query
    """

    def __init__(self,
                 bclient=get_client('gene')):
        """
        Constructor
        """
        self._cache = {}
        self._bclient = bclient

    def _query_mygene(self, val):
        """
        Queries biothings_client with 'val' to find
        hit

        :param val: id to send to :py:mod:`biothings_client`
        :type val: string
        :return: gene symbol or None
        :rtype: string
        """
        try:
            res = self._bclient.query(val)
            if res is None:
                logger.debug('Got None back from query for: ' + val)
                return ''
            logger.debug('Result from query for ' + val + ' ' + str(res))
            if res['total'] == 0:
                logger.debug('Got No hits back from query for: ' + val)
                return ''
            if len(res['hits']) > 0:
                logger.debug('Got a hit from query for: ' + val)
                sym_name = res['hits'][0].get('symbol')
                if sym_name is None:
                    logger.debug('Symbol name was None for ' + val)
                    return ''
                return sym_name.upper()
        except HTTPError as he:
            logger.error('Caught exception running query for: ' + val)

        return None

    def _query_uniprot(self, val):
        """

        :param val:
        :return:
        """
        res = requests.get('https://www.uniprot.org/uniprot/' +
                           val.upper() + '.txt')
        for entry in res.text.split('\n'):
            if entry.startswith('GN'):
                if 'Name' in entry:
                    return re.sub(';.*', '',
                                  re.sub(' .*', '',
                                         re.sub('^GN.*Name=',
                                                '', entry)))
        return None

    def get_symbol(self, val):
        """
        Queries biothings_client with 'val' to find
        hit

        :param val: id to send to :py:mod:`biothings_client`
        :type val: string
        :return: gene symbol or None
        :rtype: string
        """
        if val is None:
            logger.error('None passed in')
            return None

        cache_symbol = self._cache.get(val)
        if cache_symbol is not None:
            if cache_symbol == '':
                return None
            return cache_symbol

        sym_name = self._query_mygene(val)
        if sym_name is None or sym_name == '':
            res = self._query_uniprot(val)
            if res is not None:
                self._cache[val] = res
                return res
            sym_name = ''
        self._cache[val] = sym_name
        return sym_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ndex` module."""

import tempfile
import shutil
import os
import unittest
import json
from unittest.mock import MagicMock
from unittest import mock
import requests
import requests_mock

from ndexutil.ndex import NDExExtraUtils
from ndexutil.exceptions import NDExUtilError
from ndex2.nice_cx_network import NiceCXNetwork


class TestNDExExtraUtils(unittest.TestCase):
    """
        Tests NDExExtraUtils in ndex module
    """
    def setUp(self):
        """Set up test fixtures, if any."""
        pass

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def test_get_node_id_mapping_from_node_attribute_none_cx_file(self):
        util = NDExExtraUtils()
        try:
            util.get_node_id_mapping_from_node_attribute(cxfile=None)
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('cxfile is None', str(ne))

    def test_get_node_id_mapping_from_node_attribute_no_cx_file(self):
        util = NDExExtraUtils()
        temp_dir = tempfile.mkdtemp()
        try:
            nofile = os.path.join(temp_dir, 'doesnotexist.cx')
            util.get_node_id_mapping_from_node_attribute(cxfile=nofile)
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual(nofile + ' file not found', str(ne))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_node_id_mapping_from_node_attribute_no_nodeid_attr(self):
        util = NDExExtraUtils()
        temp_dir = tempfile.mkdtemp()
        try:
            cxfile = os.path.join(temp_dir, 'foo.cx')
            net = NiceCXNetwork()
            net.create_node('node1')
            net.create_node('node2')
            net.create_node('node3')
            with open(cxfile, 'w') as f:
                json.dump(net.to_cx(), f)
            res = util.get_node_id_mapping_from_node_attribute(cxfile=cxfile)
            self.assertEqual({}, res)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_node_id_mapping_from_node_attribute_success(self):
        util = NDExExtraUtils()
        temp_dir = tempfile.mkdtemp()
        try:
            cxfile = os.path.join(temp_dir, 'foo.cx')
            net = NiceCXNetwork()
            net.create_node('node1')
            net.create_node('node2')
            net.create_node('node3')
            with open(cxfile, 'w') as f:
                json.dump(net.to_cx(), f)
            util.add_node_id_as_node_attribute(cxfile=cxfile,
                                               outcxfile=cxfile)
            res = util.get_node_id_mapping_from_node_attribute(cxfile=cxfile)
            self.assertEqual({0: 0, 1: 1, 2: 2}, res)
        finally:
            shutil.rmtree(temp_dir)

    def test_update_network_on_ndex_invalid_args(self):
        util = NDExExtraUtils()
        try:
            util.update_network_on_ndex()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('NDEx client is None', str(ne))

        try:
            util.update_network_on_ndex(client=MagicMock())
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Network UUID is None', str(ne))

        try:
            util.update_network_on_ndex(client=MagicMock(),
                                        networkid='1234')
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('cxfile is None', str(ne))

        temp_dir = tempfile.mkdtemp()
        try:
            try:
                non_exist_file = os.path.join(temp_dir,
                                              'doesnotexist')
                util.update_network_on_ndex(client=MagicMock(),
                                            networkid='1234',
                                            cxfile=non_exist_file)
                self.fail('Expected NDExUtilError')
            except NDExUtilError as ne:
                self.assertEqual(non_exist_file +
                                 ' is not a file', str(ne))
        finally:
            shutil.rmtree(temp_dir)

    def test_update_network_on_ndex_success(self):
        util = NDExExtraUtils()
        temp_dir = tempfile.mkdtemp()
        try:
            cxfile = os.path.join(temp_dir, 'some.cx')
            with open(cxfile, 'wb') as f:
                f.write(b'hello')
            client = MagicMock()
            client.update_cx_network = MagicMock(return_value='response')
            res = util.update_network_on_ndex(client=client,
                                              networkid='1234',
                                              cxfile=cxfile)
            self.assertEqual('response', res)
            client.update_cx_network.assert_called_with(mock.ANY,
                                                        '1234')
        finally:
            shutil.rmtree(temp_dir)

    def test_update_network_aspect_on_ndex_invalid_args(self):
        util = NDExExtraUtils()
        try:
            util.update_network_aspect_on_ndex()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('NDEx client is None', str(ne))

        try:
            util.update_network_aspect_on_ndex(client=MagicMock())
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Network UUID is None', str(ne))

        try:
            util.update_network_aspect_on_ndex(client=MagicMock(),
                                               networkid='1234')
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Aspect name is None', str(ne))

        try:
            util.update_network_aspect_on_ndex(client=MagicMock(),
                                               networkid='1234',
                                               aspect_name='name')
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Aspect data is None', str(ne))

    def test_update_network_aspect_on_ndex_success(self):
        util = NDExExtraUtils()
        temp_dir = tempfile.mkdtemp()
        try:
            client = MagicMock()
            client.put = MagicMock(return_value='response')
            cart_aspect = [{'node': 0, 'x': 2, 'y': 3}]
            res = util.update_network_aspect_on_ndex(client=client,
                                                     networkid='1234',
                                                     aspect_name='cartesianLayout',
                                                     aspect_data=cart_aspect)
            self.assertEqual('response', res)
            net = NiceCXNetwork()
            net.set_opaque_aspect('cartesianLayout', cart_aspect)
            client.put.assert_called_with('/network/1234/aspects',
                                                        put_json=json.dumps(net.to_cx()))
        finally:
            shutil.rmtree(temp_dir)

    def test_download_network_from_ndex_invalid_args(self):
        util = NDExExtraUtils()
        try:
            util.download_network_from_ndex()
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('NDEx client is None', str(ne))

        try:
            util.download_network_from_ndex(client=MagicMock())
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Network UUID is None', str(ne))

        try:
            util.download_network_from_ndex(client=MagicMock(),
                                            networkid=MagicMock())
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Destfile is None', str(ne))

    def test_download_network_from_ndex_invalidfile(self):
        temp_dir = tempfile.mkdtemp()
        util = NDExExtraUtils()
        client = MagicMock()
        try:
            destfile = temp_dir
            with requests_mock.Mocker() as m:
                m.get('http://foo.com', content=b'hello')
                client.get_network_as_cx_stream = MagicMock(return_value=requests.get('http://foo.com', stream=True))
                try:
                    util.download_network_from_ndex(client=client,
                                                    networkid='1234',
                                                    destfile=destfile)
                    self.fail('Expected an error')
                except Exception as e:
                    pass

            client.get_network_as_cx_stream.assert_called_with('1234')
        finally:
            shutil.rmtree(temp_dir)

    def test_download_network_from_ndex_success(self):
        temp_dir = tempfile.mkdtemp()
        util = NDExExtraUtils()
        client = MagicMock()
        try:
            destfile = os.path.join(temp_dir, 'foo.cx')
            with requests_mock.Mocker() as m:
                m.get('http://foo.com', content=b'hello')
                client.get_network_as_cx_stream = MagicMock(return_value=requests.get('http://foo.com', stream=True))
                res = util.download_network_from_ndex(client=client,
                                                      networkid='1234',
                                                      destfile=destfile)
                self.assertTrue(os.path.isfile(res))
                with open(res, 'rb') as f:
                    self.assertEqual(b'hello', f.read())
            client.get_network_as_cx_stream.assert_called_with('1234')
        finally:
            shutil.rmtree(temp_dir)

    def test_extract_layout_aspect_from_cx_success(self):
        util = NDExExtraUtils()
        temp_dir = tempfile.mkdtemp()
        try:
            cxfile = os.path.join(temp_dir, 'foo.cx')
            net = NiceCXNetwork()
            nid = net.create_node('node1')
            net.add_node_attribute(property_of=nid,
                                   name=NDExExtraUtils.ORIG_NODE_ID_ATTR,
                                   values=4,
                                   type='long')
            nid = net.create_node('node2')
            net.add_node_attribute(property_of=nid,
                                   name=NDExExtraUtils.ORIG_NODE_ID_ATTR,
                                   values=5,
                                   type='long')
            nid = net.create_node('node3')
            net.add_node_attribute(property_of=nid,
                                   name=NDExExtraUtils.ORIG_NODE_ID_ATTR,
                                   values=6,
                                   type='long')
            net.set_opaque_aspect('cartesianLayout', [{'node': 0, 'x': 1, 'y': 2},
                                                      {'node': 1, 'x': 3, 'y': 4},
                                                      {'node': 2, 'x': 5, 'y': 6, 'z': 7}])
            with open(cxfile, 'w') as f:
                json.dump(net.to_cx(), f)
            res = util.get_node_id_mapping_from_node_attribute(cxfile=cxfile)
            self.assertEqual({0: 4, 1: 5, 2: 6}, res)

            res = util.extract_layout_aspect_from_cx(input_cx_file=cxfile)
            self.assertEqual([{'node': 4, 'x': 1.0, 'y': 2.0},
                              {'node': 5, 'x': 3.0, 'y': 4.0},
                              {'node': 6, 'x': 5.0, 'y': 6.0, 'z': 7.0}], res)
        finally:
            shutil.rmtree(temp_dir)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ndexmisctools` module."""

import tempfile
import shutil
import os
import uuid
import json
from json.decoder import JSONDecodeError
from requests.exceptions import HTTPError
import unittest
from unittest.mock import MagicMock

import ndex2
from ndexutil.exceptions import ConfigError
from ndexutil.exceptions import NDExUtilError
from ndexutil.ndexmisctools import TSVLoader
from ndexutil.config import NDExUtilConfig


class Params(object):
    pass


class TestTSVLoader(unittest.TestCase):
    """
        Tests TSVLoader in ndexmisctools module
    """

    def get_gene_disease_style_cx_file(self):
        return os.path.join(os.path.dirname(__file__),
                            'tsv', 'gene-disease-style.cx')

    def get_dummy_params(self):
        p = Params()
        p.username = 'bob'
        p.password = 'password'
        p.server = 'ndex'
        p.layout = None
        return p

    def setUp(self):
        """Set up test fixtures, if any."""
        pass

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def test_parse_config_no_credentials_needed_from_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = MagicMock()
            p.username = 'bob'
            p.password = 'password'
            p.server = 'public.ndexbio.org'
            p.conf = temp_dir
            loader = TSVLoader(p)
            self.assertEqual('bob', loader._user)
            self.assertEqual('password', loader._pass)
            self.assertEqual('public.ndexbio.org', loader._server)
        finally:
            shutil.rmtree(temp_dir)

    def test_parse_config_no_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = MagicMock()
            p.username = 'bob'
            p.password = '-'
            p.server = 'public.ndexbio.org'
            p.conf = temp_dir
            TSVLoader(p)
            self.fail('Expected ConfigError')
        except ConfigError as e:
            self.assertEqual('No configuration file found', str(e))
        finally:
            shutil.rmtree(temp_dir)

    def test_parse_config_valid_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            cfile = os.path.join(temp_dir, 'configfile')
            with open(cfile, 'w') as f:
                f.write('[foo]\n')
                f.write(NDExUtilConfig.USER + ' = theuser\n')
                f.write(NDExUtilConfig.PASSWORD + ' = thepass\n')
                f.write(NDExUtilConfig.SERVER + ' = theserver\n')

            # try with username set to - to load it from config
            p = MagicMock()
            p.username = '-'
            p.password = 'somepass'
            p.server = 'ndex'
            p.conf = cfile
            p.profile = 'foo'
            loader = TSVLoader(p)
            self.assertEqual('theuser', loader._user)
            self.assertEqual('somepass', loader._pass)
            self.assertEqual('ndex', loader._server)

            # try with password set to - to load it from config
            p.password = '-'
            loader = TSVLoader(p)
            self.assertEqual('theuser', loader._user)
            self.assertEqual('thepass', loader._pass)
            self.assertEqual('ndex', loader._server)

            # try with server set to - to load it from config
            p.server = '-'
            loader = TSVLoader(p)
            self.assertEqual('theuser', loader._user)
            self.assertEqual('thepass', loader._pass)
            self.assertEqual('theserver', loader._server)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_client_with_alt_set(self):
        p = self.get_dummy_params()
        loader = TSVLoader(p, altclient='foo')
        self.assertEqual('foo', loader._get_client())

    def test_get_cx_style_template_not_set(self):
        p = self.get_dummy_params()
        p.t = None
        loader = TSVLoader(p)
        self.assertEqual(None, loader._get_cx_style())

    def test_get_cx_style_template_is_invalid_cx_not_in_jsonformat(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()

            invalidcx = os.path.join(temp_dir, 'foo.cx')
            with open(invalidcx, 'w') as f:
                f.write('hi')

            p.t = invalidcx
            loader = TSVLoader(p)
            loader._get_cx_style()
            self.fail('Expected JSONDecodeError')
        except JSONDecodeError:
            pass
        finally:
            shutil.rmtree(temp_dir)

    def test_get_cx_style_template_is_invalid_cx(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()

            invalidcx = os.path.join(temp_dir, 'foo.cx')
            with open(invalidcx, 'w') as f:
                f.write('{"hi": "there"}')

            p.t = invalidcx
            loader = TSVLoader(p)
            net = loader._get_cx_style()
            self.assertEqual(0, len(net.get_nodes()))
            self.assertEqual(0, len(net.get_edges()))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_cx_style_template_is_invalid_uuid(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()

            p.t = str(uuid.uuid4()) + str(uuid.uuid4())
            loader = TSVLoader(p)
            loader._get_cx_style()
            self.fail('Expected NDExUtilError ')
        except NDExUtilError as e:
            self.assertEqual(p.t +
                             ' does not appear to be a valid NDEx UUID',
                             str(e))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_stripped_down_style_cx_from_server_raises_httperrors(self):
        p = self.get_dummy_params()
        p.t = str(uuid.uuid4())
        mockclient = MagicMock()
        httperror = HTTPError(MagicMock(status=404), 'Client Error')
        mockclient.get_network_aspect_as_cx_stream =\
            MagicMock(side_effect=[httperror, httperror, httperror])
        loader = TSVLoader(p)
        try:
            loader._get_stripped_down_style_cx_from_server(mockclient)
            self.fail('Expected exception')
        except NDExUtilError as ne:
            self.assertTrue('No style found on template' in str(ne))

    def test_get_stripped_down_style_cx_from_server_legacy_cx(self):
        p = self.get_dummy_params()
        p.t = str(uuid.uuid4())
        mockclient = MagicMock()
        httperror = HTTPError(MagicMock(status=404), 'Client Error')

        net_attribs = [{"n": "author", "v": "xxx"},
                       {"n": "name", "v": "foo2"},
                       {"n": "description", "v": "hellothere"}]
        mocknetres = MagicMock()
        mocknetres.text = json.dumps(net_attribs)

        vis_attribs = [{"p": "network"}]
        mockvisres = MagicMock()
        mockvisres.text = json.dumps(vis_attribs)
        mockclient.get_network_aspect_as_cx_stream =\
            MagicMock(side_effect=[mocknetres, httperror, mockvisres])
        loader = TSVLoader(p)
        try:
            net = loader._get_stripped_down_style_cx_from_server(mockclient)
            self.assertEqual('foo2', net.get_name())
            self.assertEqual(vis_attribs,
                             net.get_opaque_aspect('cyVisualProperties'))
        except NDExUtilError as ne:
            self.assertTrue('No style found on template' in str(ne))

    def test_get_cx_style_valid_cx_file(self):
        p = self.get_dummy_params()

        p.t = self.get_gene_disease_style_cx_file()
        loader = TSVLoader(p)
        net = loader._get_cx_style()
        self.assertEqual(5, len(net.get_nodes()))

    def test_get_cx_style_valid_network_file(self):
        p = self.get_dummy_params()
        p.t = str(uuid.uuid4())
        mockclient = MagicMock()

        net_attribs = [{"n": "@context", "v": "{\"signor\": \"http://foo\""},
                       {"n": "author", "v": "xxx"},
                       {"n": "name", "v": "foo2"},
                       {"n": "description", "v": "hellothere"}]
        mocknetres = MagicMock()
        mocknetres.text = json.dumps(net_attribs)

        vis_attribs = [{"p": "network"}]
        mockvisres = MagicMock()

        mockvisres.text = json.dumps(vis_attribs)

        mockclient.get_network_aspect_as_cx_stream =\
            MagicMock(side_effect=[mocknetres, mockvisres])
        loader = TSVLoader(p)

        net = loader._get_cx_style(client=mockclient)
        self.assertEqual(0, len(net.get_nodes()))
        self.assertEqual('foo2', net.get_name())

    def test_get_network_attributes_network_is_none(self):
        p = self.get_dummy_params()
        p.description = None
        p.name = None
        p.copyattribs = True
        loader = TSVLoader(p)
        net_attribs = loader._get_network_attributes(None)
        self.assertEqual(None, net_attribs)

        # set name
        p.name = 'hello'
        loader = TSVLoader(p)
        net_attribs = loader._get_network_attributes(None)
        self.assertEqual(1, len(net_attribs))
        self.assertTrue({'n': 'name', 'v': 'hello',
                         'd': 'string'} in net_attribs)

        # now set description too
        p.description = 'something'
        loader = TSVLoader(p)
        net_attribs = loader._get_network_attributes(None)
        self.assertEqual(2, len(net_attribs))
        self.assertTrue({'n': 'name', 'v': 'hello',
                         'd': 'string'} in net_attribs)
        self.assertTrue({'n': 'description', 'v': 'something',
                         'd': 'string'} in net_attribs)

    def test_get_network_attributes_with_network(self):
        p = self.get_dummy_params()
        p.description = None
        p.name = None
        p.t = self.get_gene_disease_style_cx_file()
        p.copyattribs = True
        loader = TSVLoader(p)
        net = loader._get_cx_style()
        net_attribs = loader._get_network_attributes(net)

        self.assertEqual(8, len(net_attribs))
        self.assertTrue({'n': 'description',
                         'v': 'Used to style CTD '
                              'gene-disease associations'} in net_attribs)
        self.assertTrue({'n': 'name',
                         'v': 'Style for CTD '
                              'Gene-disease associations'} in net_attribs)

        # set name
        p.name = 'hello'
        loader = TSVLoader(p)
        net_attribs = loader._get_network_attributes(net)
        self.assertEqual(8, len(net_attribs))
        self.assertTrue({'n': 'name', 'v': 'hello',
                         'd': 'string'} in net_attribs)
        self.assertTrue({'n': 'name',
                         'v': 'Style for CTD '
                              'Gene-disease associations'} not in net_attribs)

        # now set description too
        p.description = 'something'
        loader = TSVLoader(p)
        net_attribs = loader._get_network_attributes(net)
        self.assertEqual(8, len(net_attribs))
        self.assertTrue({'n': 'name', 'v': 'hello',
                         'd': 'string'} in net_attribs)
        self.assertTrue({'n': 'description', 'v': 'something',
                         'd': 'string'} in net_attribs)
        self.assertTrue({'n': 'description',
                         'v': 'Used to style CTD '
                              'gene-disease associations'} not in net_attribs)

    def test_get_network_attributes_with_network_with_context_set(self):
        p = self.get_dummy_params()
        p.description = None
        p.name = None
        p.t = 'abase'
        p.copyattribs = True
        loader = TSVLoader(p)

        cx = [{"numberVerification": [{"longNumber": 281474976710655}]},
              {"metaData": [{"name": "nodes", "elementCount": 2,
                             "idCounter": 220,
                             "version": "1.0", "properties": []},
                            {"name": "networkAttributes", "elementCount": 9,
                             "version": "1.0", "properties": []},
                            {"name": "edges", "elementCount": 1,
                             "idCounter": 221, "version": "1.0",
                             "properties": []}]},
              {"nodes": [{"@id": 220, "n": "alpha"},
                         {"@id": 219, "n": "omega"}]},
              {"networkAttributes": [{"n": "@context",
                                      "v": "{\"hi\": \"bye\"}"},
                                     {"n": "organism", "v": "z"},
                                     {"n": "description", "v": "hhh"},
                                     {"n": "version", "v": "za"},
                                     {"n": "reference", "v": "z"},
                                     {"n": "name", "v": "small"}]},
              {"edges": [{"@id": 221, "s": 220, "t": 219,
                          "i": "interacts with"}]},
              {"status": [{"error": "", "success": True}]}]

        net = ndex2.create_nice_cx_from_raw_cx(cx)

        net_attribs = loader._get_network_attributes(net)
        self.assertEqual(5, len(net_attribs))
        for n_a in net_attribs:
            if n_a['n'] == '@context':
                self.fail('@context should have been removed, '
                          'but here it is: ' + str(n_a))

    def test_upload_network_with_new_network(self):
        p = self.get_dummy_params()
        p.u = None
        mockclient = MagicMock()
        mockclient.save_cx_stream_as_new_network = MagicMock(return_value='hi')
        loader = TSVLoader(p)
        res = loader._upload_network(mockclient,
                                     self.get_gene_disease_style_cx_file())
        self.assertEqual(0, res)

    def test_upload_network_update_network(self):
        p = self.get_dummy_params()
        p.u = 'hello'
        mockclient = MagicMock()
        mockclient.update_cx_network = MagicMock(return_value='hi')
        loader = TSVLoader(p)
        res = loader._upload_network(mockclient,
                                     self.get_gene_disease_style_cx_file())
        self.assertEqual(0, res)

    def test_upload_network_with_invalid_credentials(self):
        p = self.get_dummy_params()
        p.u = None
        mockclient = MagicMock()
        mockclient.save_cx_stream_as_new_network =\
            MagicMock(side_effect=HTTPError('401 Client Error'))
        loader = TSVLoader(p)
        res = loader._upload_network(mockclient,
                                     self.get_gene_disease_style_cx_file())
        self.assertEqual(2, res)

    def test_upload_network_with_some_other_error(self):
        p = self.get_dummy_params()
        p.u = None
        mockclient = MagicMock()
        mockclient.save_cx_stream_as_new_network =\
            MagicMock(side_effect=HTTPError('500 Server Error'))
        loader = TSVLoader(p)
        res = loader._upload_network(mockclient,
                                     self.get_gene_disease_style_cx_file())
        self.assertEqual(3, res)

    def test_get_tsvfile_header_is_none(self):
        p = self.get_dummy_params()
        p.tsv_file = 'sometemplate.cx'
        p.t = None
        p.uppercaseheader = False
        p.header = None
        loader = TSVLoader(p)
        self.assertEqual(p.tsv_file, loader._get_tsvfile())

    def test_get_tsvfile_header_is_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()
            cxfile = os.path.join(temp_dir, 'some.cx')
            with open(cxfile, 'w') as f:
                f.write('hello\n')
            p.t = cxfile
            p.tsv_file = cxfile
            p.tmpdir = temp_dir
            p.header = 'blahblah'
            loader = TSVLoader(p)
            loader._tmpdir = temp_dir
            tmptsv = os.path.join(temp_dir, 'temp.tsv')
            self.assertEqual(tmptsv,
                             loader._get_tsvfile())
            with open(tmptsv, 'r') as f:
                self.assertEqual('blahblah\n', f.readline())
                self.assertEqual('hello\n', f.readline())
                self.assertEqual('', f.read())
        finally:
            shutil.rmtree(temp_dir)

    def test_get_tsvfile_uppercaseheader_is_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()
            cxfile = os.path.join(temp_dir, 'some.cx')
            with open(cxfile, 'w') as f:
                f.write('myheader\n')
                f.write('hello\n')
            p.t = cxfile
            p.tsv_file = cxfile
            p.tmpdir = temp_dir
            p.header = None
            p.uppercaseheader = True
            loader = TSVLoader(p)
            loader._tmpdir = temp_dir
            tmptsv = os.path.join(temp_dir, 'temp.tsv')
            self.assertEqual(tmptsv,
                             loader._get_tsvfile())
            with open(tmptsv, 'r') as f:
                self.assertEqual('MYHEADER\n', f.readline())
                self.assertEqual('hello\n', f.readline())
                self.assertEqual('', f.read())
        finally:
            shutil.rmtree(temp_dir)

    def test_mock_run_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()
            p.tsv_file = 'somefile.tsv'
            with open(p.tsv_file, 'w') as f:
                f.write('a\tb\n')
                f.write('c\td\n')
            p.t = self.get_gene_disease_style_cx_file()
            p.u = None
            p.header = None
            p.name = 'new name'
            p.tmpdir = temp_dir
            p.uppercaseheader = False
            p.description = 'new description'
            p.load_plan = 'plan'
            p.outputcx = None
            p.skipupload = None
            p.copyattribs = True
            mockclient = MagicMock()
            mockclient.save_cx_stream_as_new_network =\
                MagicMock(return_value='hi')
            mockfac = MagicMock()
            mocktsvloader = MagicMock()
            mocktsvloader.write_cx_network = MagicMock(return_value=None)

            mockfac.get_tsv_streamloader =\
                MagicMock(return_value=mocktsvloader)

            loader = TSVLoader(p, altclient=mockclient, streamtsvfac=mockfac)
            loader.run()

            mockfac.get_tsv_streamloader.assert_called_once()
            self.assertEqual('plan',
                             mockfac.get_tsv_streamloader.call_args[0][0])
            mocktsvloader.write_cx_network.assert_called_once()
            n_a = mocktsvloader.write_cx_network.call_args[1]
            self.assertTrue({'n': 'name',
                             'v': 'new name',
                             'd': 'string'} in n_a['network_attributes'])
            mockclient.save_cx_stream_as_new_network.assert_called_once()
            for entry in os.listdir(temp_dir):
                full_path = os.path.join(entry)
                if os.path.isdir(full_path):
                    self.fail('The run() method should have deleted '
                              'this directory: ' + full_path)

        finally:
            shutil.rmtree(temp_dir)

    def test_mock_run_success_outputcx_and_skipupload_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = self.get_dummy_params()
            p.t = self.get_gene_disease_style_cx_file()
            p.tsv_file = 'somefile.tsv'
            with open(p.tsv_file, 'w') as f:
                f.write('a\tb\n')
                f.write('c\td\n')
            p.header = None
            p.name = 'new name'
            p.uppercaseheader = False
            p.tmpdir = temp_dir
            p.description = 'new description'
            p.load_plan = 'plan'
            p.outputcx = os.path.join(temp_dir, 'my.cx')
            p.skipupload = True
            p.copyattribs = True
            mockclient = MagicMock()
            mockfac = MagicMock()
            mocktsvloader = MagicMock()
            mocktsvloader.write_cx_network = MagicMock(return_value=None)

            mockfac.get_tsv_streamloader = \
                MagicMock(return_value=mocktsvloader)

            loader = TSVLoader(p, altclient=mockclient, streamtsvfac=mockfac)
            loader.run()
            mockfac.get_tsv_streamloader.assert_called_once()
            self.assertEqual('plan',
                             mockfac.get_tsv_streamloader.call_args[0][0])
            mocktsvloader.write_cx_network.assert_called_once()
            n_a = mocktsvloader.write_cx_network.call_args[1]
            self.assertTrue({'n': 'name',
                             'v': 'new name',
                             'd': 'string'} in n_a['network_attributes'])
            for entry in os.listdir(temp_dir):
                full_path = os.path.join(entry)
                if os.path.isdir(full_path):
                    self.fail('The run() method should have deleted '
                              'this directory: ' + full_path)
            self.assertTrue(os.path.isfile(p.outputcx))
        finally:
            shutil.rmtree(temp_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ndexmisctools` module."""

import tempfile
import shutil
import os
import json
import unittest
from unittest.mock import MagicMock
from requests.exceptions import HTTPError
import ndex2
from ndex2.nice_cx_network import NiceCXNetwork
from ndexutil.exceptions import ConfigError
from ndexutil.exceptions import NDExUtilError
from ndexutil.ndexmisctools import StyleUpdator
from ndexutil.config import NDExUtilConfig


class TestStyleUpdator(unittest.TestCase):
    """
        Tests StyleUpdate in ndexmisctools module
    """
    def setUp(self):
        """Set up test fixtures, if any."""
        pass

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def get_dummy_configfile(self, temp_dir):
        """
        Given a base temp directory create a config file
        :param temp_dir:
        :return: full path to the config file
        """
        cfile = os.path.join(temp_dir, 'configfile')
        with open(cfile, 'w') as f:
            f.write('[foo]\n')
            f.write(NDExUtilConfig.USER + ' = theuser\n')
            f.write(NDExUtilConfig.PASSWORD + ' = thepass\n')
            f.write(NDExUtilConfig.SERVER + ' = theserver\n')
        return cfile

    def test_parse_config_no_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = MagicMock()
            p.username = 'bob'
            p.password = '-'
            p.server = 'public.ndexbio.org'
            p.conf = temp_dir
            remover = StyleUpdator(p)
            remover._parse_config()
            self.fail('Expected ConfigError')
        except ConfigError as e:
            self.assertEqual('No configuration file found', str(e))
        finally:
            shutil.rmtree(temp_dir)

    def test_parse_config_valid_configfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            cfile = self.get_dummy_configfile(temp_dir)

            # try with username set to - to load it from config
            p = MagicMock()
            p.conf = cfile
            p.profile = 'foo'
            loader = StyleUpdator(p)
            loader._parse_config()
            self.assertEqual('theuser', loader._user)
            self.assertEqual('thepass', loader._pass)
            self.assertEqual('theserver', loader._server)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_client_with_alt_set(self):
        p = MagicMock()
        adder = StyleUpdator(p, altclient='foo')
        self.assertEqual('foo', adder._get_client())

    def test_get_style_function_with_style_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            p = MagicMock()
            p.stylefile = os.path.join(temp_dir, 'foo.cx')
            p.newcopy = False
            net = NiceCXNetwork()
            net.set_name('foo')
            with open(p.stylefile, 'w') as f:
                json.dump(net.to_cx(), f)

            loader = StyleUpdator(p)
            loader._get_style_function_to_use()
            self.assertEqual('foo', loader._style.get_name())
            sf = loader._get_style_function_to_use()
            self.assertEqual(type(loader._apply_style_from_file_to_new_network),
                             type(sf))
            self.assertTrue('_apply_style_from_file_'
                            'to_original_network' in str(sf))

            # try with copy
            loader._args.newcopy = True
            loader._get_style_function_to_use()
            self.assertEqual('foo', loader._style.get_name())
            sf = loader._get_style_function_to_use()
            self.assertEqual(type(loader._apply_style_from_file_to_new_network),
                             type(sf))
            self.assertTrue('_apply_style_from_file_to_new_network' in str(sf))

        finally:
            shutil.rmtree(temp_dir)

    def test_get_style_function_with_no_style_file(self):
        p = MagicMock()
        p.stylefile = None
        p.newcopy = False

        loader = StyleUpdator(p)
        loader._get_style_function_to_use()
        sf = loader._get_style_function_to_use()
        self.assertEqual(type(loader._apply_style_from_file_to_new_network),
                         type(sf))
        self.assertTrue('_apply_style_from_uuid_to_original_network' in str(sf))

        # try with copy
        loader._args.newcopy = True
        loader._get_style_function_to_use()
        sf = loader._get_style_function_to_use()
        self.assertEqual(type(loader._apply_style_from_file_to_new_network),
                         type(sf))
        self.assertTrue('_apply_style_from_uuid_to_new_network' in str(sf))

    def test_copy_networkset_works_retry_exceeded(self):
        p = MagicMock()
        p.networksetretry = 1
        loader = StyleUpdator(p)
        loader._client = MagicMock()
        loader._client.\
            get_networkset = MagicMock(return_value={'name': 'foo',
                                                     'description': 'desc'})
        loader._client.\
            create_networkset = MagicMock(side_effect=HTTPError('err'))

        try:
            loader._copy_networkset('netsetid')
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue('After ' in str(ne))

        loader._client.get_networkset.assert_called_once_with('netsetid')
        loader._client. \
            create_networkset.assert_called_once_with('Copy of foo', 'desc')

    def test_copy_networkset_works_first_try(self):
        p = MagicMock()
        p.networksetretry = 5
        loader = StyleUpdator(p)
        loader._client = MagicMock()
        loader._client.\
            get_networkset = MagicMock(return_value={'name': 'foo',
                                                     'description': 'desc'})

        loader._client.\
            create_networkset = MagicMock(return_value='http://foo/'
                                                       'network/uuid')
        loader._old_to_new = {'1': '2',
                              '2': '3'}
        loader._client.add_networks_to_networkset = MagicMock()
        res = loader._copy_networkset('netsetid')
        self.assertEqual('uuid', res)
        loader._client.get_networkset.assert_called_once_with('netsetid')
        loader._client. \
            create_networkset.assert_called_once_with('Copy of foo', 'desc')

        loader._client.add_networks_to_networkset.assert_called_once_with('uuid',
                                                                          ['2', '3'])








# -*- coding: utf-8 -*-

import os
import logging
import ndexutil
import json
import requests
import pandas
from ndexutil.argparseutil import ArgParseFormatter
from ndexutil.config import NDExUtilConfig

from ndexutil.exceptions import NDExUtilError
from ndex2.client import Ndex2
import ndex2

# create logger
logger = logging.getLogger('ndexutil.reports')


class FeaturedNetworkReportCommand(object):
    """
    Updates network in NDEx with Networkx layout
    """
    COMMAND = 'featurednetworkreport'
    UUID_KEY = 'UUID'
    TYPE_KEY = 'Type'
    TITLE_KEY = 'FeaturedNetworkTitle'
    OWNER_KEY = 'Owner'
    NET_UUID_KEY = 'NetworkUUID'
    NUM_NODES_KEY = 'NumberNodes'
    NUM_EDGES_KEY = 'NumberEdges'
    HAS_LAYOUT_KEY = 'HasLayout'
    NET_NAME_KEY = 'NetworkName'

    HEADER_LIST = [UUID_KEY, TYPE_KEY, TITLE_KEY, OWNER_KEY, NET_UUID_KEY,
                   NUM_NODES_KEY, NUM_EDGES_KEY, HAS_LAYOUT_KEY, NET_NAME_KEY]

    def __init__(self, theargs):
        """
        Constructor
        :param theargs: command line arguments ie theargs.name theargs.type
        """
        self._args = theargs
        self._server = self._args.server

    def _parse_config(self):
        """
        Parses config extracting the following fields:
        :py:const:`~ndexutil.config.NDExUtilConfig.USER`
        :py:const:`~ndexutil.config.NDExUtilConfig.PASSWORD`
        :py:const:`~ndexutil.config.NDExUtilConfig.SERVER`
        :return: None
        """
        if self._server != '-':
            return

        ncon = NDExUtilConfig(conf_file=self._args.conf)
        con = ncon.get_config()

        if self._server == '-':
            self._server = con.get(self._args.profile, NDExUtilConfig.SERVER)

    def _get_client(self):
        """
        Gets Ndex2 client
        :return: Ndex2 python client
        :rtype: :py:class:`~ndex2.client.Ndex2`
        """
        return Ndex2(self._server)

    def run(self):
        """
        :raises NDExUtilError if there is an error
        :return: number of attributes updated upon success
        """

        logger.warning('THIS IS AN UNTESTED ALPHA IMPLEMENTATION '
                       'AND MAY CONTAIN ERRORS')

        self._parse_config()
        client = self._get_client()
        feat_nets = self.get_featured_networks_as_json()
        report_dict = dict()
        for key in FeaturedNetworkReportCommand.HEADER_LIST:
            report_dict[key] = []

        if 'items' not in feat_nets:
            raise NDExUtilError('Expected data under '
                                '\'items\' dict, but none found')

        for entry in feat_nets['items']:
            if 'type' not in entry:
                logger.error(str(entry) +
                             ' no \'type\' found for entry. Skipping')
                continue
            if entry['type'].lower() == 'user':
                self.add_user_networks_to_report(client=client,
                                                 report_dict=report_dict,
                                                 featured_entry=entry)
            elif entry['type'].lower() == 'network':
                self.add_network_to_report(client=client,
                                           report_dict=report_dict,
                                           net_uuid=entry['UUID'],
                                           featured_entry=entry)
            elif entry['type'].lower() == 'networkset':
                self.add_networkset_to_report(client=client,
                                              report_dict=report_dict,
                                              featured_entry=entry,
                                              networkset_id=entry['UUID'])
            else:
                logger.error('Unknown type: ' +
                             str(entry['type']) +
                             ' Skipping')

        df = pandas.DataFrame.from_dict(report_dict)
        logger.info('Resulting CSV size: ' + str(df.shape))
        df.to_csv(self._args.output)

    def append_featured_info(self, report_dict=None, featured_entry=None):
        """

        :param report_dict:
        :param featured_entry:
        :return:
        """
        report_dict[FeaturedNetworkReportCommand.UUID_KEY].append(featured_entry['UUID'])
        report_dict[FeaturedNetworkReportCommand.TYPE_KEY].append(featured_entry['type'])
        report_dict[FeaturedNetworkReportCommand.TITLE_KEY].append(featured_entry['title'])

    def add_networkset_to_report(self, client=None,
                                 report_dict=None,
                                 featured_entry=None,
                                 networkset_id=None):
        """

        :param client:
        :param report_dict:
        :param featured_entry:
        :param networkset_id:
        :return:
        """
        res = client.get_networkset(networkset_id)
        for net_set_id in res['networks']:
            self.add_network_to_report(client=client, report_dict=report_dict,
                                       featured_entry=featured_entry,
                                       net_uuid=net_set_id)

    def add_user_networks_to_report(self, client=None,
                                    report_dict=None,
                                    featured_entry=None):
        """

        :param report_dict:
        :param featured_entry:
        :return:
        """

        res = client.post('/search/user', json.dumps({'searchString': featured_entry['UUID']}))
        if not isinstance(res, dict):
            logger.error('Error trying to get user from id: ' + featured_entry['UUID'])
            return
        if 'resultList' not in res:
            logger.error('Expected \'resultList\' in dict, but not found: ' +
                         str(res))
            return
        if 'userName' not in res['resultList'][0]:
            logger.error('Expected \'userName\' under \'resultList\', but '
                         'not found: ' + str(res['resultList'][0]))
            return
        user_name = res['resultList'][0]['userName']
        logger.info('Looking at networks owned by user: ' + user_name)
        res = client.post('/search/network?size=10000', json.dumps({'searchString': '',
                                                                    'accountName': user_name}))
        if 'numFound' in res and res['numFound'] > 0:
            for sub_ent in res['networks']:
                self.add_network_to_report(client=client,report_dict=report_dict,
                                           featured_entry=featured_entry,
                                           summary=sub_ent)

        # gotta check for networksets for user now
        res = client.get('/user/' + featured_entry['UUID'] + '/networksets')
        for net_set in res:
            counter = 0
            for id_for_net in net_set['networks']:
                self.add_network_to_report(client=client, report_dict=report_dict,
                                           featured_entry=featured_entry,
                                           net_uuid=id_for_net)
                if counter > 500:
                    logger.warning('Only first 500 networks of ' +
                                   str(len(net_set['networks'])) +
                                   ' networks in : ' + featured_entry['title'] +
                                   ' will be checked: ')
                    break
                counter += 1

    def add_network_to_report(self, client=None,
                              report_dict=None,
                              featured_entry=None,
                              net_uuid=None,
                              summary=None):
        """

        :param client:
        :param report_dict:
        :param featured_entry:
        :param net_uuid:
        :param summary:
        :return:
        """
        if summary is None:
            summary = client.get_network_summary(net_uuid)

        if self._args.excludehaslayout is True:
            if summary['hasLayout'] is True:
                logger.debug('Skipping since --excludehaslayout flag is set')
                return True
        report_dict[FeaturedNetworkReportCommand.OWNER_KEY].append(summary['owner'])
        report_dict[FeaturedNetworkReportCommand.NET_UUID_KEY].append(summary['externalId'])
        report_dict[FeaturedNetworkReportCommand.NUM_NODES_KEY].append(summary['nodeCount'])
        report_dict[FeaturedNetworkReportCommand.NUM_EDGES_KEY].append(summary['edgeCount'])
        report_dict[FeaturedNetworkReportCommand.HAS_LAYOUT_KEY].append(summary['hasLayout'])
        report_dict[FeaturedNetworkReportCommand.NET_NAME_KEY].append(summary['name'])

        self.append_featured_info(report_dict=report_dict,
                                  featured_entry=featured_entry)
        return True

    def get_featured_networks_as_json(self):
        """

        :return:
        """
        if os.path.isfile(self._args.featuredjson):
            with open(self._args.featuredjson, 'r') as f:
                return json.load(f)

        if not self._args.featuredjson.startswith('http'):
            raise NDExUtilError(self._args.featuredjson +
                                ' is not a file and does '
                                'not start with http')
        res = requests.get(self._args.featuredjson)
        if res.status_code != 200:
            raise NDExUtilError('Received error status code when trying'
                                'to retreive featured network json: ' +
                                str(res.status_code))
        return res.json()


    @staticmethod
    def add_subparser(subparsers):
        """
        adds a subparser
        :param subparsers:
        :return:
        """
        desc = """

Version {version}

The {cmd} command queries NDEx for featured networks and generates
a summary report on those networks in following format:

{header}

Example invocation:

ndexmisctools.py {cmd} public.ndexbio.org 

WARNING: THIS IS AN UNTESTED ALPHA IMPLEMENTATION AND MAY CONTAIN
         ERRORS. YOU HAVE BEEN WARNED.

        """.format(version=ndexutil.__version__,
                   cmd=FeaturedNetworkReportCommand.COMMAND,
                   header=','.join(FeaturedNetworkReportCommand.HEADER_LIST))

        parser = subparsers.add_parser(FeaturedNetworkReportCommand.COMMAND,
                                       help='Generates report on Featured '
                                            'networks',
                                       description=desc,
                                       formatter_class=ArgParseFormatter)
        parser.add_argument('server', help='NDEx server, if set to - then '
                                           'value from config will be used')
        parser.add_argument('--featuredjson',
                            default='https://home.ndexbio.org/landing_page_'
                                    'content/v2_4_2/featured_networks.json',
                            help='URL to featured networks json file OR'
                                 'path to local json file')
        parser.add_argument('--output',
                            help='Path to write CSV file')
        parser.add_argument('--excludehaslayout', action='store_true',
                            help='If set, omit networks that have a layout')
        return parser



# -*- coding: utf-8 -*-


import argparse


class ArgParseFormatter(argparse.ArgumentDefaultsHelpFormatter,
                        argparse.RawDescriptionHelpFormatter):
    """
    Combines argparse formatter with default values output with
    raw description formatter
    """
    pass